@@ -22,6 +22,7 @@
 #include "nm-default.h"
 
 #include "nm-fortisslvpn-editor.h"
+#include "nm-fortissl-properties.h"
 
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -40,6 +41,7 @@ typedef struct {
 	gboolean window_added;
 	gboolean new_connection;
 	gchar *trusted_cert;
+	gchar *realm;
 	NMSettingSecretFlags otp_flags;
 } FortisslvpnEditorPrivate;
 
@@ -53,26 +55,6 @@ G_DEFINE_TYPE_EXTENDED (FortisslvpnEditor, fortisslvpn_editor, G_TYPE_OBJECT, 0,
 
 /*****************************************************************************/
 
-static gboolean
-check_validity (FortisslvpnEditor *self, GError **error)
-{
-	FortisslvpnEditorPrivate *priv = FORTISSLVPN_EDITOR_GET_PRIVATE (self);
-	GtkWidget *widget;
-	const char *str;
-
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "gateway_entry"));
-	str = gtk_entry_get_text (GTK_ENTRY (widget));
-	if (!str || !strlen (str)) {
-		g_set_error (error,
-		             NMV_EDITOR_PLUGIN_ERROR,
-		             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
-		             NM_FORTISSLVPN_KEY_GATEWAY);
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 static void
 stuff_changed_cb (GtkWidget *widget, gpointer user_data)
 {
@@ -172,23 +154,28 @@ advanced_dialog_response_cb (GtkWidget *dialog, gint response, gpointer user_dat
 {
 	FortisslvpnEditor *self = FORTISSLVPN_EDITOR (user_data);
 	FortisslvpnEditorPrivate *priv = FORTISSLVPN_EDITOR_GET_PRIVATE (self);
-	GtkEntry *entry = GTK_ENTRY (gtk_builder_get_object (priv->builder, "trusted_cert_entry"));
-	GtkToggleButton *use_otp = GTK_TOGGLE_BUTTON (gtk_builder_get_object (priv->builder, "use_otp"));
+	GtkEntry *trusted_cert_entry = GTK_ENTRY (gtk_builder_get_object (priv->builder, "trusted_cert_entry"));
+	GtkEntry *realm_entry = GTK_ENTRY (gtk_builder_get_object (priv->builder, "realm_entry"));
+	GtkSwitch *use_otp = GTK_SWITCH (gtk_builder_get_object (priv->builder, "use_otp"));
+
+	g_return_if_fail (trusted_cert_entry);
+	g_return_if_fail (realm_entry);
 
-	g_assert (entry);
 	if (response == GTK_RESPONSE_OK) {
 		g_free (priv->trusted_cert);
-		priv->trusted_cert = g_strdup (gtk_entry_get_text (entry));
+		priv->trusted_cert = g_strdup (gtk_entry_get_text (trusted_cert_entry));
+		priv->realm = g_strdup (gtk_entry_get_text (realm_entry));
 		stuff_changed_cb (NULL, self);
 
-		if (gtk_toggle_button_get_active (use_otp))
+		if (gtk_switch_get_active (use_otp))
 			priv->otp_flags |= NM_SETTING_SECRET_FLAG_NOT_SAVED;
 		else
 			priv->otp_flags &= ~NM_SETTING_SECRET_FLAG_NOT_SAVED;
 	} else {
-		gtk_entry_set_text (entry, priv->trusted_cert);
-		gtk_toggle_button_set_active (use_otp,
-		                              priv->otp_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED);
+		gtk_entry_set_text (trusted_cert_entry, priv->trusted_cert);
+		gtk_entry_set_text (realm_entry, priv->realm);
+		gtk_switch_set_active (use_otp,
+		                       priv->otp_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED);
 	}
 
 	gtk_widget_hide (dialog);
@@ -229,8 +216,8 @@ init_editor_plugin (FortisslvpnEditor *self, NMConnection *connection, GError **
 	priv->group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "gateway_entry"));
-	if (!widget)
-		return FALSE;
+	g_return_val_if_fail (widget, FALSE);
+
 	gtk_size_group_add_widget (priv->group, widget);
 	if (s_vpn) {
 		value = nm_setting_vpn_get_data_item (s_vpn, NM_FORTISSLVPN_KEY_GATEWAY);
@@ -240,8 +227,8 @@ init_editor_plugin (FortisslvpnEditor *self, NMConnection *connection, GError **
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "user_entry"));
-	if (!widget)
-		return FALSE;
+	g_return_val_if_fail (widget, FALSE);
+
 	gtk_size_group_add_widget (priv->group, widget);
 	if (s_vpn) {
 		value = nm_setting_vpn_get_data_item (s_vpn, NM_FORTISSLVPN_KEY_USER);
@@ -251,8 +238,7 @@ init_editor_plugin (FortisslvpnEditor *self, NMConnection *connection, GError **
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (stuff_changed_cb), self);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "trusted_cert_entry"));
-	if (!widget)
-		return FALSE;
+	g_return_val_if_fail (widget, FALSE);
 	if (s_vpn) {
 		priv->trusted_cert = g_strdup (nm_setting_vpn_get_data_item (s_vpn,
 		                                                             NM_FORTISSLVPN_KEY_TRUSTED_CERT));
@@ -261,15 +247,25 @@ init_editor_plugin (FortisslvpnEditor *self, NMConnection *connection, GError **
 		gtk_entry_set_text (GTK_ENTRY (widget), priv->trusted_cert);
 	}
 
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "realm_entry"));
+	g_return_val_if_fail (widget, FALSE);
+	if (s_vpn) {
+		priv->realm = g_strdup (nm_setting_vpn_get_data_item (s_vpn,
+		                                                      NM_FORTISSLVPN_KEY_REALM));
+		if (!priv->realm)
+			priv->realm = g_strdup ("");
+		gtk_entry_set_text (GTK_ENTRY (widget), priv->realm);
+	}
+
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "use_otp"));
-	if (!widget)
-		return FALSE;
+	g_return_val_if_fail (widget, FALSE);
+
 	if (s_vpn) {
 		nm_setting_get_secret_flags (NM_SETTING (s_vpn),
 		                             NM_FORTISSLVPN_KEY_OTP, &priv->otp_flags,
 		                             NULL);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
-		                              priv->otp_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED);
+		gtk_switch_set_active (GTK_SWITCH (widget),
+		                       priv->otp_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED);
 	}
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "show_passwords_checkbutton"));
@@ -293,8 +289,8 @@ init_editor_plugin (FortisslvpnEditor *self, NMConnection *connection, GError **
 	                    "user_password_entry");
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "ca_chooser"));
-	if (!widget)
-		return FALSE;
+	g_return_val_if_fail (widget, FALSE);
+
 	gtk_size_group_add_widget (priv->group, widget);
 	if (s_vpn) {
 		value = nm_setting_vpn_get_data_item (s_vpn, NM_FORTISSLVPN_KEY_CA);
@@ -304,8 +300,8 @@ init_editor_plugin (FortisslvpnEditor *self, NMConnection *connection, GError **
 	g_signal_connect (G_OBJECT (widget), "update-preview", G_CALLBACK (stuff_changed_cb), self);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "cert_chooser"));
-	if (!widget)
-		return FALSE;
+	g_return_val_if_fail (widget, FALSE);
+
 	gtk_size_group_add_widget (priv->group, widget);
 	if (s_vpn) {
 		value = nm_setting_vpn_get_data_item (s_vpn, NM_FORTISSLVPN_KEY_CERT);
@@ -315,8 +311,8 @@ init_editor_plugin (FortisslvpnEditor *self, NMConnection *connection, GError **
 	g_signal_connect (G_OBJECT (widget), "update-preview", G_CALLBACK (stuff_changed_cb), self);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "key_chooser"));
-	if (!widget)
-		return FALSE;
+	g_return_val_if_fail (widget, FALSE);
+
 	gtk_size_group_add_widget (priv->group, widget);
 	if (s_vpn) {
 		value = nm_setting_vpn_get_data_item (s_vpn, NM_FORTISSLVPN_KEY_KEY);
@@ -326,14 +322,14 @@ init_editor_plugin (FortisslvpnEditor *self, NMConnection *connection, GError **
 	g_signal_connect (G_OBJECT (widget), "update-preview", G_CALLBACK (stuff_changed_cb), self);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "advanced_dialog"));
-	if (!widget)
-		return FALSE;
+	g_return_val_if_fail (widget, FALSE);
+
 	g_signal_connect (G_OBJECT (widget), "response", G_CALLBACK (advanced_dialog_response_cb), self);
 	g_signal_connect (G_OBJECT (widget), "delete-event", G_CALLBACK (advanced_dialog_delete_cb), NULL);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "advanced_button"));
-	if (!widget)
-		return FALSE;
+	g_return_val_if_fail (widget, FALSE);
+
 	g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (advanced_button_clicked_cb), self);
 
 	return TRUE;
@@ -388,10 +384,6 @@ update_connection (NMVpnEditor *iface,
 	NMSettingVpn *s_vpn;
 	GtkWidget *widget;
 	const char *str;
-	gboolean valid = FALSE;
-
-	if (!check_validity (self, error))
-		return FALSE;
 
 	s_vpn = NM_SETTING_VPN (nm_setting_vpn_new ());
 	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_DBUS_SERVICE_FORTISSLVPN, NULL);
@@ -438,13 +430,21 @@ update_connection (NMVpnEditor *iface,
 		                              NM_FORTISSLVPN_KEY_TRUSTED_CERT,
 		                              priv->trusted_cert);
 
+	/* Realm */
+	if (priv->realm && strlen (priv->realm))
+		nm_setting_vpn_add_data_item (s_vpn,
+		                              NM_FORTISSLVPN_KEY_REALM,
+		                              priv->realm);
+
 	/* Use OTP */
 	nm_setting_set_secret_flags (NM_SETTING (s_vpn), NM_FORTISSLVPN_KEY_OTP, priv->otp_flags, NULL);
 
-	nm_connection_add_setting (connection, NM_SETTING (s_vpn));
-	valid = TRUE;
+	if (!nm_fortisslvpn_properties_validate (s_vpn, error))
+		return FALSE;
 
-	return valid;
+	nm_connection_add_setting (connection, NM_SETTING (s_vpn));
+
+	return TRUE;
 }
 
 static void
@@ -484,7 +484,7 @@ nm_fortisslvpn_editor_new (NMConnection *connection, GError **error)
 		g_return_val_if_reached (NULL);
 	}
 
-	priv->widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "fortisslvpn-vbox"));
+	priv->widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "fortisslvpn_grid"));
 	if (!priv->widget) {
 		g_set_error (error, NMV_EDITOR_PLUGIN_ERROR, 0, "could not load UI widget");
 		g_object_unref (object);
