@@ -39,6 +39,7 @@
 
 #include <glib/gstdio.h>
 
+#include "nm-fortissl-properties.h"
 #include "nm-ppp-status.h"
 #include "nm-fortisslvpn-pppd-service-dbus.h"
 #include "nm-utils/nm-shared-utils.h"
@@ -86,34 +87,6 @@ typedef struct {
 #define NM_FORTISSLVPN_WAIT_PPPD 10000 /* 10 seconds */
 #define FORTISSLVPN_SERVICE_SECRET_TRIES "fortisslvpn-service-secret-tries"
 
-typedef struct {
-	const char *name;
-	GType type;
-	bool required:1;
-} ValidProperty;
-
-/*****************************************************************************/
-
-static const ValidProperty valid_properties[] = {
-	{ NM_FORTISSLVPN_KEY_GATEWAY,           G_TYPE_STRING, TRUE },
-	{ NM_FORTISSLVPN_KEY_USER,              G_TYPE_STRING, TRUE },
-	{ NM_FORTISSLVPN_KEY_CA,                G_TYPE_STRING, FALSE },
-	{ NM_FORTISSLVPN_KEY_TRUSTED_CERT,      G_TYPE_STRING, FALSE },
-	{ NM_FORTISSLVPN_KEY_CERT,              G_TYPE_STRING, FALSE },
-	{ NM_FORTISSLVPN_KEY_KEY,               G_TYPE_STRING, FALSE },
-	{ NM_FORTISSLVPN_KEY_PASSWORD"-flags",  G_TYPE_UINT,   FALSE },
-	{ NM_FORTISSLVPN_KEY_OTP"-flags",       G_TYPE_UINT,   FALSE },
-	{ NULL }
-};
-
-static const ValidProperty valid_secrets[] = {
-	{ NM_FORTISSLVPN_KEY_PASSWORD,          G_TYPE_STRING, TRUE },
-	{ NM_FORTISSLVPN_KEY_OTP,               G_TYPE_STRING, TRUE },
-	{ NULL }
-};
-
-/*****************************************************************************/
-
 #define _NMLOG(level, ...) \
     G_STMT_START { \
          if (gl.log_level >= (level)) { \
@@ -136,191 +109,6 @@ _LOGD_enabled (void)
 
 /*****************************************************************************/
 
-static gboolean
-validate_gateway (const char *gateway)
-{
-	if (!gateway || !strlen (gateway) || !isalnum (*gateway))
-		return FALSE;
-
-	return TRUE;
-}
-
-/* This is a bit half-assed. We should check that the user doesn't
- * abuse this to access files he ordinarily shouldn't, but we can't do
- * any better than this for we don't have any information about the
- * identity of the user that activates the connection.
- * We should probably get the certificate inline or something. */
-static gboolean
-validate_ca (const char *ca)
-{
-	struct stat ca_stat;
-
-	/* Tolerate only absolute paths */
-	if (!ca || !strlen (ca) || *ca != '/')
-		return FALSE;
-
-	if (stat (ca, &ca_stat) == -1)
-		return FALSE;
-
-	/* Allow only ordinary files */
-	if (!(ca_stat.st_mode & S_IFREG))
-		return FALSE;
-
-	/* Allow only world-readable files */
-	if ((ca_stat.st_mode & 0444) != 0444)
-		return FALSE;
-
-	return TRUE;
-}
-
-typedef struct ValidateInfo {
-	const ValidProperty *table;
-	GError **error;
-	gboolean have_items;
-} ValidateInfo;
-
-static void
-validate_one_property (const char *key, const char *value, gpointer user_data)
-{
-	ValidateInfo *info = (ValidateInfo *) user_data;
-	int i;
-
-	if (*(info->error))
-		return;
-
-	info->have_items = TRUE;
-
-	/* 'name' is the setting name; always allowed but unused */
-	if (!strcmp (key, NM_SETTING_NAME))
-		return;
-
-	for (i = 0; info->table[i].name; i++) {
-		ValidProperty prop = info->table[i];
-
-		if (strcmp (prop.name, key))
-			continue;
-
-		switch (prop.type) {
-		case G_TYPE_STRING:
-			if (   !strcmp (prop.name, NM_FORTISSLVPN_KEY_GATEWAY)
-			    && !validate_gateway (value)) {
-				g_set_error (info->error,
-				             NM_VPN_PLUGIN_ERROR,
-				             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-				             _("invalid gateway “%s”"),
-				             value);
-				return;
-			} else if (   !strcmp (prop.name, NM_FORTISSLVPN_KEY_CA)
-			           && !validate_ca (value)) {
-				g_set_error (info->error,
-				             NM_VPN_PLUGIN_ERROR,
-				             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-				             _("invalid certificate authority “%s”"),
-				             value);
-				return;
-			}
-			return; /* valid */
-		case G_TYPE_UINT:
-			errno = 0;
-			(void) strtol (value, NULL, 10);
-			if (errno == 0)
-				return; /* valid */
-
-			g_set_error (info->error,
-			             NM_VPN_PLUGIN_ERROR,
-			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			             _("invalid integer property “%s”"),
-			             key);
-			break;
-		case G_TYPE_BOOLEAN:
-			if (!strcmp (value, "yes") || !strcmp (value, "no"))
-				return; /* valid */
-
-			g_set_error (info->error,
-			             NM_VPN_PLUGIN_ERROR,
-			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			             _("invalid boolean property “%s” (not yes or no)"),
-			             key);
-			break;
-		default:
-			g_set_error (info->error,
-			             NM_VPN_PLUGIN_ERROR,
-			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			             _("unhandled property “%s” type %s"),
-			             key, g_type_name (prop.type));
-			break;
-		}
-	}
-
-	/* Did not find the property from valid_properties or the type did not match */
-	if (!info->table[i].name) {
-		g_set_error (info->error,
-		             NM_VPN_PLUGIN_ERROR,
-		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-		             _("property “%s” invalid or not supported"),
-		             key);
-	}
-}
-
-static gboolean
-validate_properties (NMSettingVpn *s_vpn, GError **error)
-{
-	ValidateInfo info = { &valid_properties[0], error, FALSE };
-	int i;
-
-	nm_setting_vpn_foreach_data_item (s_vpn, validate_one_property, &info);
-	if (!info.have_items) {
-		g_set_error (error,
-		             NM_VPN_PLUGIN_ERROR,
-		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-		             "%s",
-		             _("No VPN configuration options."));
-		return FALSE;
-	}
-
-	if (*error)
-		return FALSE;
-
-	/* Ensure required properties exist */
-	for (i = 0; valid_properties[i].name; i++) {
-		ValidProperty prop = valid_properties[i];
-		const char *value;
-
-		if (!prop.required)
-			continue;
-
-		value = nm_setting_vpn_get_data_item (s_vpn, prop.name);
-		if (!value || !strlen (value)) {
-			g_set_error (error,
-			             NM_VPN_PLUGIN_ERROR,
-			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			             _("Missing required option “%s”."),
-			             prop.name);
-			return FALSE;
-		}
-	}
-
-	return TRUE;
-}
-
-static gboolean
-validate_secrets (NMSettingVpn *s_vpn, GError **error)
-{
-	ValidateInfo info = { &valid_secrets[0], error, FALSE };
-
-	nm_setting_vpn_foreach_secret (s_vpn, validate_one_property, &info);
-	if (!info.have_items) {
-		g_set_error (error,
-		             NM_VPN_PLUGIN_ERROR,
-		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-		             "%s",
-		             _("No VPN secrets!"));
-		return FALSE;
-	}
-
-	return *error ? FALSE : TRUE;
-}
-
 static gboolean
 ensure_killed (gpointer data)
 {
@@ -477,6 +265,12 @@ run_openfortivpn (NMFortisslvpnPlugin *plugin, NMSettingVpn *s_vpn, GError **err
 	g_ptr_array_add (argv, (gpointer) g_strdup ("--pppd-plugin"));
 	g_ptr_array_add (argv, (gpointer) g_strdup (NM_FORTISSLVPN_PPPD_PLUGIN));
 
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_FORTISSLVPN_KEY_REALM);
+	if (value) {
+		g_ptr_array_add (argv, (gpointer) g_strdup ("--realm"));
+		g_ptr_array_add (argv, (gpointer) g_strdup (value));
+	}
+
 	g_ptr_array_add (argv, NULL);
 
 	_LOGD ("start %s", (str_tmp = g_strjoinv (" ", (char **) argv->pdata)));
@@ -579,22 +373,24 @@ real_connect (NMVpnServicePlugin *plugin, NMConnection *connection, GError **err
 	NMSettingVpn *s_vpn;
 	mode_t old_umask;
 	gchar *config;
-	const char *username, *password, *otp;
+	const char *username, *password, *realm, *otp;
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
 	s_vpn = NM_SETTING_VPN (nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN));
 	g_assert (s_vpn);
 
-	if (!validate_properties (s_vpn, error))
+	if (!nm_fortisslvpn_properties_validate (s_vpn, error))
 		return FALSE;
 
-	if (!validate_secrets (s_vpn, error))
+	if (!nm_fortisslvpn_properties_validate_secrets (s_vpn, error))
 		return FALSE;
 
 	if (!get_credentials (s_vpn, &username, &password, &otp, error))
 		return FALSE;
 
+	realm = nm_setting_vpn_get_data_item (s_vpn, NM_FORTISSLVPN_KEY_REALM);
+
 	g_clear_object (&priv->connection);
 	priv->connection = g_object_ref (connection);
 
@@ -607,8 +403,12 @@ real_connect (NMVpnServicePlugin *plugin, NMConnection *connection, GError **err
 	 * secrets on the command line */
 	priv->config_file = g_strdup_printf (NM_FORTISSLVPN_STATEDIR "/%s.config",
 	                                     nm_connection_get_uuid (connection));
-	config = g_strdup_printf ("username = %s\npassword = %s%s%s\n",
+	config = g_strdup_printf ("username = %s\n"
+	                          "password = %s"
+	                          "%s%s"
+	                          "%s%s\n",
 	                          username, password,
+	                          realm ? "\nrealm = " : "", realm ? realm : "",
 	                          otp ? "\notp = " : "", otp ? otp : "");
 	old_umask = umask (0077);
 	if (!g_file_set_contents (priv->config_file, config, -1, error)) {
